import configparser
import os
import logging

class AwsConfigConst:
    IOT_PORT_DEFAULT = 443
    IOT_CA_FILE_DEFAULT = "/cert/AmazonRootCA1.pem"
    IOT_CERT_FILE_DEFAULT = "/cert/device_cert.pem"
    IOT_PKCS11_PATH_DEFAULT = "/usr/lib/plug-and-trust/libsss_pkcs11.so"
    IOT_KEY_LABEL_DEFAULT = "sss:100100F0"
    IOT_POLICY_FILE_DEFAULT = "/config/aws_iot_policy.json"

class AwsConfig:
    CLOUD_NAME = "AWS"
    IOT_SHADOW_ENDPOINT = "AWS_IOT_SHADOW_ENDPOINT"
    IOT_PORT = "AWS_IOT_PORT"
    IOT_CA_FILE = "AWS_IOT_CA_FILE"
    IOT_CERT_FILE = "AWS_IOT_CERT_FILE"
    IOT_PKCS11_PATH = "AWS_IOT_PKCS11_PATH"
    IOT_KEY_LABEL = "AWS_IOT_KEY_LABEL"
    IOT_PIN = "AWS_IOT_PIN"
    ACCESS_KEY = "AWS_ACCESS_KEY"
    SECRET_KEY = "AWS_SECRET_KEY"
    IOT_HOST = "AWS_IOT_HOST"
    IOT_REGION = "AWS_IOT_REGION"
    IOT_ENDPOINT = "AWS_IOT_ENDPOINT"
    IOT_ACCOUNTID = "AWS_IOT_ACCOUNTID"
    IOT_POLICY_FILE = "AWS_IOT_POLICY_FILE"

    def __init__(self, config_path):
        self._configs = {}
        self._isConfigValid = True
        self._configpath = config_path
        self._logging = logging.getLogger('cloud-agent')

    def get_and_valid_data(self, config_data, dict_data, name, str=True, error=True, default=None, file=False, valid=True):
        if str:
            dict_data[name] = config_data.get(name)
        else:
            dict_data[name] = config_data.getint(name)

        if not valid:
            return

        if not dict_data[name]:
            if error:
                self._logging.error("%s is not found.", name)
                self._isConfigValid = False
            else:
                self._logging.warning("%s is not found.", name)
                dict_data[name] = default

        if file:
           if not os.path.isfile(dict_data[name]):
               self._logging.error("%s %s is not existed.", name, dict_data[name])
               self._isConfigValid = False

    def load(self, config_data):
        # Shadow
        shadow_conf = {}
        self.get_and_valid_data(config_data, shadow_conf, AwsConfig.IOT_SHADOW_ENDPOINT)
        self.get_and_valid_data(config_data, shadow_conf, AwsConfig.IOT_PORT,
                                str=False, error=False, default=AwsConfigConst.IOT_PORT_DEFAULT)
        self.get_and_valid_data(config_data, shadow_conf, AwsConfig.IOT_CA_FILE,
                                error=False, default=AwsConfigConst.IOT_CA_FILE_DEFAULT, file=True)
        self.get_and_valid_data(config_data, shadow_conf, AwsConfig.IOT_CERT_FILE,
                                error=False, default=AwsConfigConst.IOT_CERT_FILE_DEFAULT, file=True)
        self.get_and_valid_data(config_data, shadow_conf, AwsConfig.IOT_PKCS11_PATH,
                                error=False, default=AwsConfigConst.IOT_PKCS11_PATH_DEFAULT)
        self.get_and_valid_data(config_data, shadow_conf, AwsConfig.IOT_KEY_LABEL,
                                error=False, default=AwsConfigConst.IOT_KEY_LABEL_DEFAULT)
        self.get_and_valid_data(config_data, shadow_conf, AwsConfig.IOT_PIN,
                                error=False, default="", valid=False)
        self._configs['shadow'] = shadow_conf

        # Prov
        prov_conf = {}
        self.get_and_valid_data(config_data, prov_conf, AwsConfig.ACCESS_KEY, error=False)
        self.get_and_valid_data(config_data, prov_conf, AwsConfig.SECRET_KEY, error=False)
        self.get_and_valid_data(config_data, prov_conf, AwsConfig.IOT_ACCOUNTID, error=False)

        if (prov_conf[AwsConfig.ACCESS_KEY] and
            prov_conf[AwsConfig.SECRET_KEY] and
            prov_conf[AwsConfig.IOT_ACCOUNTID]):
            prov_conf_error = True
        else:
            self._logging.info("Config parameter required for provisioning are not set.")
            prov_conf_error = False

        self.get_and_valid_data(config_data, prov_conf, AwsConfig.IOT_HOST, error=prov_conf_error)
        self.get_and_valid_data(config_data, prov_conf, AwsConfig.IOT_REGION, error=prov_conf_error)
        self.get_and_valid_data(config_data, prov_conf, AwsConfig.IOT_ENDPOINT, error=prov_conf_error)

        self.get_and_valid_data(config_data, prov_conf, AwsConfig.IOT_POLICY_FILE,
                                error=False, default=AwsConfigConst.IOT_POLICY_FILE_DEFAULT, file=True)
        self.get_and_valid_data(config_data, prov_conf, AwsConfig.IOT_CERT_FILE,
                                error=False, default=AwsConfigConst.IOT_CERT_FILE_DEFAULT, file=True)
        self._configs['prov'] = prov_conf

    def clear_prov_conf(self, clear_list):
        config = configparser.ConfigParser()
        config.optionxform = str
        config.read(self._configpath)

        for item in clear_list:
            if self._configs['prov'][item]:
                config[AwsConfig.CLOUD_NAME][item] = ""

        with open(self._configpath, "w", encoding="utf-8") as configfile:
            config.write(configfile, True)

    def get_shadow_conf(self):
        return self._configs['shadow']

    def get_prov_conf(self):
        return self._configs['prov']

    def is_valid(self):
        return self._isConfigValid

#
# End of File
#
